/* -*- C++ -*-
 *
 * This file is a part of LEMON, a generic C++ optimization library
 *
 * Copyright (C) 2003-2006
 * Egervary Jeno Kombinatorikus Optimalizalasi Kutatocsoport
 * (Egervary Research Group on Combinatorial Optimization, EGRES).
 *
 * Permission to use, modify and distribute this software is granted
 * provided that this copyright notice appears in all copies. For
 * precise terms see the accompanying LICENSE file.
 *
 * This software is provided "AS IS" with no warranty of any kind,
 * express or implied, and with no claim as to its suitability for any
 * purpose.
 *
 */

#ifndef LEMON_LP_GLPK_H
#define LEMON_LP_GLPK_H

///\file
///\brief Header of the LEMON-GLPK lp solver interface.
///\ingroup gen_opt_group

#include <lemon/lp_base.h>
extern "C" {
#include <glpk.h>
}

namespace lemon {


  /// \brief Interface for the GLPK LP solver
  /// 
  /// This class implements an interface for the GLPK LP solver.
  ///\ingroup gen_opt_group
  class LpGlpk : virtual public LpSolverBase {
  protected:
    LPX* lp;
    
  public:
    
    typedef LpSolverBase Parent;
    
    LpGlpk();
    LpGlpk(const LpGlpk &);
    ~LpGlpk();
    
  protected:
    virtual LpSolverBase &_newLp();
    virtual LpSolverBase &_copyLp();

    virtual int _addCol();
    virtual int _addRow();
    virtual void _eraseCol(int i);
    virtual void _eraseRow(int i);
    virtual void _getColName(int col,       std::string & name);
    virtual void _setColName(int col, const std::string & name);
    virtual void _setRowCoeffs(int i, LpRowIterator b, LpRowIterator e);
    virtual void _setColCoeffs(int i, LpColIterator b, LpColIterator e);
    virtual void _setCoeff(int row, int col, Value value);
    virtual Value _getCoeff(int row, int col);

    virtual void _setColLowerBound(int i, Value value);
    virtual Value _getColLowerBound(int i);
    virtual void _setColUpperBound(int i, Value value);
    virtual Value _getColUpperBound(int i);

//     virtual void _setRowLowerBound(int i, Value value);
//     virtual void _setRowUpperBound(int i, Value value);
    virtual void _setRowBounds(int i, Value lower, Value upper);
    virtual void _getRowBounds(int i, Value &lb, Value &ub);
    virtual void _setObjCoeff(int i, Value obj_coef);
    virtual Value _getObjCoeff(int i);
    virtual void _clearObj();
//     virtual void _setObj(int length,
//                          int  const * indices, 
//                          Value  const * values ) = 0;

    ///\e
    
    ///\todo It should be clarified
    ///
    virtual SolveExitStatus _solve();
    virtual Value _getPrimal(int i);
    virtual Value _getDual(int i);
    virtual Value _getPrimalValue();
    virtual bool _isBasicCol(int i);
    ///\e
    
    ///\todo It should be clarified
    ///
    virtual SolutionStatus _getPrimalStatus();
    virtual SolutionStatus _getDualStatus();
    virtual ProblemTypes _getProblemType();

    virtual void _setMax();
    virtual void _setMin();

    virtual bool _isMax();

  public:
    ///Set the verbosity of the messages

    ///Set the verbosity of the messages
    ///
    ///\param m is the level of the messages output by the solver routines.
    ///The possible values are:
    ///- 0 --- no output (default value)
    ///- 1 --- error messages only
    ///- 2 --- normal output
    ///- 3 --- full output (includes informational messages)
    void messageLevel(int m);
    ///Turns on or off the presolver

    ///Turns on (\c b is \c true) or off (\c b is \c false) the presolver
    ///
    ///The presolver is off by default.
    void presolver(bool b);

    LPX *lpx() {return lp;}
  };
} //END OF NAMESPACE LEMON

#endif //LEMON_LP_GLPK_H

